/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * File Name          : freertos.c
  * Description        : Code for freertos applications
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"
#include "main.h"
#include "cmsis_os.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */     
#include "i2c.h"
#include "sai.h"
#include "a5130.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
#define BUFF_SIZE       1024

#define A7131_PKG_LEN 63
#define FRAME_LEN (A7131_PKG_LEN*2)/3

typedef enum
{
  SPI_BUFF_1 = 0x00,
  SPI_BUFF_2 = 0x01
} SPIBuffIndex_t;

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
/* USER CODE BEGIN Variables */
uint8_t cmbBuff[BUFF_SIZE];
uint8_t audioBuff[BUFF_SIZE];
uint8_t txBuff[BUFF_SIZE];

volatile SPIBuffIndex_t spiBuffIdx;
volatile SPIBuffIndex_t curBuffIdx;
volatile uint8_t rxBuff1[BUFF_SIZE];
volatile uint8_t rxBuff2[BUFF_SIZE];
__IO int16_t UpdatePointer = -1;
/* USER CODE END Variables */
osThreadId SPIHandle;
uint32_t SPIBuffer[ 128 ];
osStaticThreadDef_t SPIControlBlock;
osThreadId SAIHandle;
uint32_t SAIBuffer[ 128 ];
osStaticThreadDef_t SAIControlBlock;
osSemaphoreId rxSemHandle;
osStaticSemaphoreDef_t rxSemCBlock;

/* Private function prototypes -----------------------------------------------*/
/* USER CODE BEGIN FunctionPrototypes */
void CS4265_init(void);
void LED_blink();
/* USER CODE END FunctionPrototypes */

void SPITask(void const * argument);
void SAITask(void const * argument);

void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/* GetIdleTaskMemory prototype (linked to static allocation support) */
void vApplicationGetIdleTaskMemory( StaticTask_t **ppxIdleTaskTCBBuffer, StackType_t **ppxIdleTaskStackBuffer, uint32_t *pulIdleTaskStackSize );

/* USER CODE BEGIN GET_IDLE_TASK_MEMORY */
static StaticTask_t xIdleTaskTCBBuffer;
static StackType_t xIdleStack[configMINIMAL_STACK_SIZE];
  
void vApplicationGetIdleTaskMemory( StaticTask_t **ppxIdleTaskTCBBuffer, StackType_t **ppxIdleTaskStackBuffer, uint32_t *pulIdleTaskStackSize )
{
  *ppxIdleTaskTCBBuffer = &xIdleTaskTCBBuffer;
  *ppxIdleTaskStackBuffer = &xIdleStack[0];
  *pulIdleTaskStackSize = configMINIMAL_STACK_SIZE;
  /* place for user code */
}                   
/* USER CODE END GET_IDLE_TASK_MEMORY */

/**
  * @brief  FreeRTOS initialization
  * @param  None
  * @retval None
  */
void MX_FREERTOS_Init(void) {
  /* USER CODE BEGIN Init */
       
  /* USER CODE END Init */

  /* USER CODE BEGIN RTOS_MUTEX */
  /* add mutexes, ... */
  /* USER CODE END RTOS_MUTEX */

  /* Create the semaphores(s) */
  /* definition and creation of rxSem */
  osSemaphoreStaticDef(rxSem, &rxSemCBlock);
  rxSemHandle = osSemaphoreCreate(osSemaphore(rxSem), 1);

  /* USER CODE BEGIN RTOS_SEMAPHORES */
  /* add semaphores, ... */
  /* USER CODE END RTOS_SEMAPHORES */

  /* USER CODE BEGIN RTOS_TIMERS */
  /* start timers, add new ones, ... */
  /* USER CODE END RTOS_TIMERS */

  /* USER CODE BEGIN RTOS_QUEUES */
  /* add queues, ... */
  /* USER CODE END RTOS_QUEUES */

  /* Create the thread(s) */
  /* definition and creation of SPI */
  osThreadStaticDef(SPI, SPITask, osPriorityNormal, 0, 128, SPIBuffer, &SPIControlBlock);
  SPIHandle = osThreadCreate(osThread(SPI), NULL);

  /* definition and creation of SAI */
  osThreadStaticDef(SAI, SAITask, osPriorityNormal, 0, 128, SAIBuffer, &SAIControlBlock);
  SAIHandle = osThreadCreate(osThread(SAI), NULL);

  /* USER CODE BEGIN RTOS_THREADS */
  /* add threads, ... */
  /* USER CODE END RTOS_THREADS */

}

/* USER CODE BEGIN Header_SPITask */
/**
  * @brief  Function implementing the SPI thread.
  * @param  argument: Not used 
  * @retval None
  */
/* USER CODE END Header_SPITask */
void SPITask(void const * argument)
{
  /* USER CODE BEGIN SPITask */
  uint8_t recv;

  A5130_init();
  A5130_WriteReg( FIFO1_REG,  A7131_PKG_LEN-1 ); // set FIFO length   
  
  /* Infinite loop */
  for(;;)
  {
    StrobeCmd( CMD_RX );
    while(HAL_GPIO_ReadPin(WTR_GPIO_Port,WTR_Pin));
    recv = A5130_ReadReg(MODE_REG);
      if( (recv & 0x20) == 0 )
      {
        if(spiBuffIdx==SPI_BUFF_1)
        {
          ReadFIFOData(A7131_PKG_LEN,rxBuff2);
          spiBuffIdx=SPI_BUFF_2;
        }
        else
        {
          ReadFIFOData(A7131_PKG_LEN,rxBuff1);          
          spiBuffIdx=SPI_BUFF_1;
        }
      }
      else
      {
        //CRC error handler
      }
  }
  /* USER CODE END SPITask */
}

/* USER CODE BEGIN Header_SAITask */
/**
* @brief Function implementing the SAI thread.
* @param argument: Not used
* @retval None
*/
/* USER CODE END Header_SAITask */
void SAITask(void const * argument)
{
  /* USER CODE BEGIN SAITask */
  volatile uint8_t* buf;

  memset(audioBuff,0x00,BUFF_SIZE);
  if(HAL_OK != HAL_SAI_Transmit_DMA(&hsai_BlockA1, (uint8_t *)audioBuff, FRAME_LEN))
  {
    Error_Handler();
  }

  CS4265_init();
  /* Infinite loop */
  for(;;)
  {
    osDelay(1);
  }
  /* USER CODE END SAITask */
}

/* Private application code --------------------------------------------------*/
/* USER CODE BEGIN Application */

void CS4265_init(void)
{
  uint8_t ret;
  uint8_t addr[] = {
    0x00, 0x00/*power up*/, 0x08, 0x18/*Left-Justified*/,
    0x10/*Left-Justified*/, 0x00/*MCLK Divider 1*/, 0x40, 0x00, 
    0x00, 0x19, 0x00/*max vol*/, 0x00/*max vol*/, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00
  };

  HAL_Delay(100); //wait CS4265 power up

  for(int i=2;i<0x14;i++)
  {
    if(HAL_I2C_Mem_Write(&hi2c1, (uint16_t)I2C_ADDRESS, i, I2C_MEMADD_SIZE_8BIT, &addr[i], 1, 100)!= HAL_OK)
      Error_Handler();
    if(HAL_I2C_Mem_Read(&hi2c1, (uint16_t)I2C_ADDRESS, i, I2C_MEMADD_SIZE_8BIT, &ret, 1, 100)!= HAL_OK)
      Error_Handler();
    if(ret!=addr[i])
      Error_Handler();
  }
}

void HAL_SAI_TxCpltCallback(SAI_HandleTypeDef *hsai)
{
  volatile uint8_t *buf;

  if(curBuffIdx==spiBuffIdx){
    memset(audioBuff,0x00,168*2);
  }else{
    curBuffIdx=spiBuffIdx;
    if(spiBuffIdx==SPI_BUFF_1)
      buf = rxBuff1;
    else
      buf = rxBuff2;
    for(int i=0;i<A7131_PKG_LEN;i+=3){
      memcpy(audioBuff+(i/3*8)+1,buf+i,3);
      memset(audioBuff+(i/3*8)+4,0x00,5);
    }
  }
}
/* USER CODE END Application */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
